# Investments::Journal.pm - The Object Class that provides a Investments::Journal Object
#$Id: Journal.pm,v 1.13 2005/05/11 12:35:42 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::Investments::Journal;
use strict;
use Portal::Accounting::BaseL;
use Portal::Accounting::DB::Security;
use Portal::Accounting::Objects::Investment;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.13 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Investment Journal",
  new => "New Investment",
  edit => "Edit Investment",
  delete => "Delete Investment",
  save => "Save Investment",
  shareSelect => "Select Lots",
  lotSelect => "Select Lots",
  );

=head1 NAME

Investments::Journal - Object used to work with Investments::Journal

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Journal;
  my $obj = Portal::Accounting::Investments::Journal->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Investments::Journal is the Investments::Journal Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Investments::Journal module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Investments::Journal State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;
  $self->{valids} = Portal::Accounting::Objects::Investment->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($self->{valids}->error) { $self->error($self->{valids}->errorMessage); }

  # do validation
  if (!$self->Portal::Accounting::Investments::Journal::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject c_display()

=cut
sub c_display
{
  my $self = shift;
  my %args = ( command => "display", @_ );
  my $command = $args{command};
  my $doc = $self->prepDoc;
  my $l = $self->{langObj};
  my $lc;
  my $val = $self->{valids};
  my $Buy = $val->map("buy");
  my $Sell = $val->map("sell");
  my $Fee = $val->map("fee");
  my $Memo = $val->map("memo");
  my $Merger = $val->map("acqs");
  my $Return = $val->map("rtcp");
  my $Spin = $val->map("spof");
  my $Split = $val->map("splt");
  my $StockTx = $val->map("sxfr");

  $doc->print(<<"FOO");
<p>
  Investment Transactions:<br>
  <a href="?$self->{getInfo}&command=new&type=buy">$Buy</a><br>
  <a href="?$self->{getInfo}&command=new&type=sell">$Sell</a><br>
  <a href="?$self->{getInfo}&command=new&type=fee">$Fee</a><br>
  <a href="?$self->{getInfo}&command=new&type=memo">$Memo</a><br>
  <a href="?$self->{getInfo}&command=new&type=acqs">$Merger</a><br>
  <a href="?$self->{getInfo}&command=new&type=rtcp">$Return</a><br>
  <a href="?$self->{getInfo}&command=new&type=spof">$Spin</a><br>
  <a href="?$self->{getInfo}&command=new&type=splt">$Split</a><br>
  <a href="?$self->{getInfo}&command=new&type=sxfr">$StockTx</a><br>
</p>
FOO
  if ($val->error) { $doc->print($val->error);  return $doc;  }
  return $doc;
}

sub c_new
{
  my $self = shift;
  if (exists $self->{valids}->{objects}->{$self->{input}->{type}}) { $self->{object} = "Investments::".$self->{valids}->{objects}->{$self->{input}->{type}}; }
  else
  {
    my $doc = $self->prepDoc;
    $doc->setFocus("body");
    $doc->print("<p>Invalid type sent to Investments::Journal->c_new</p>\n");
    return $doc;
  }
  return $self->SUPER::c_new(@_);
}

sub c_edit
{
  my $self = shift;
  if (exists $self->{valids}->{objects}->{$self->{input}->{type}}) { $self->{object} = "Investments::".$self->{valids}->{objects}->{$self->{input}->{type}}; }
  else
  {
    my $doc = $self->prepDoc;
    $doc->setFocus("body");
    $doc->print("<p>Invalid type sent to Investments::Journal->c_edit</p>\n");
    return $doc;
  }
  return $self->SUPER::c_edit(@_);
}

sub c_save
{
  my $self = shift;
  if (exists $self->{valids}->{objects}->{$self->{input}->{type}}) { $self->{object} = "Investments::".$self->{valids}->{objects}->{$self->{input}->{type}}; }
  else
  {
    my $doc = $self->prepDoc;
    $doc->setFocus("body");
    $doc->print("<p>Invalid type sent to Investments::Journal->c_save</p>\n");
    return $doc;
  }
  return $self->SUPER::c_save(@_);
}

sub c_lotSelect
{
  my $self = shift;
  my $date = $self->{input}->{date};
  my $account = $self->{input}->{account};
  my $subAccount = $self->{input}->{stock};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $doc = $self->prepDoc;
  $doc->setFocus("body");
  my $securityDBObject = Portal::Accounting::DB::Security->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($securityDBObject->error) { $doc->print($securityDBObject->errorMessage);  return $doc; }
  #get all lots for sub account sent in
  my @lotNodes = $securityDBObject->getLotBalanceEntriesFor(accountNum => $account, subAccountNum => $subAccount, ledgerCode => $ledgerCode, order => "lot", eventDate=>$date);
  if ($securityDBObject->didErrorOccur())
  {
    $self->{errorString} .= "<font color=\"red\">SecurityDBObject had error</font><br>\n";
    $self->error(errorString => $securityDBObject->errorMessage());
    return undef;
  }
  if (not defined @lotNodes)
  {#there were no lots or there was a problem
    # lookup the account and sub account names.
    my $accountObj = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
    $accountObj->set("id", $account);
    $accountObj->select();
    if ($accountObj->error)
    {
      $doc->print($accountObj->errorMessage);
      return $doc;
    }
    my $accountName = $accountObj->get("name");
    my $stockObj = Portal::Accounting::Objects::Stock->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
    $stockObj->set("id",$subAccount);
    $stockObj->select;
    if ($stockObj->error)
    {
      $doc->print($stockObj->errorMessage);
      return $doc;
    }
    my $subAccountName = $stockObj->get("name");
    $doc->print("<p><font color=\"red\">There were no lots found for this Account ($accountName) and Security ($subAccountName) and Ledger ($ledgerCode) and Date ($date).</font></p>");
    $doc->print("<form>\n");
    $doc->print("<input type=\"button\" value=\"Close\" onClick=\"self.close();\">\n");
    $doc->print("</form>\n");
  }
  else
  {
    $doc->print("<p>Select the lot to which to apply the adjustment.</p>");
    $doc->print("<form name=\"mainForm\">\n");
    $doc->print("<select name=\"lotSelect\">\n");
    for (my $lotCount=0; $lotCount < scalar @lotNodes; $lotCount++)
    {
      $doc->print("  <option>" . $lotNodes[$lotCount]->lot . "\n");
    }
    $doc->print("</select>\n");
    $doc->print("<input type=\"button\" value=\"Done\" onClick=\"writeLots(this);self.close();\">\n");
    $doc->print("</rform>");
  }
  $doc->setFocus("javascript");
  my $form = "mainForm";
  if (defined $self->{input}->{targetForm}) { $form = $self->{input}->{targetForm};  }
  $doc->print(<<"ENDOF");
function writeLots(me)
{
  //write data back into parent window
  var lotstring = '';
  lotstring = me.options[me.lot_select.selectedIndex].text;
  var windowref = window.opener;
  windowref.document.$form.lots.value = lotstring;
}
ENDOF
  #!!! where is this called?  make sure it uses the correct api
    return $doc;
}

sub c_shareSelect
{
  my $self = shift;
  my $date = $self->{input}->{date};
  my $account = $self->{input}->{account};
  my $subAccount = $self->{input}->{stock};
  my $doc = $self->prepDoc;
  $doc->setFocus("javascript");
  $doc->print("var shareArray = new Array();\n");
  $doc->setFocus("body");
  my $securityDBObject = Portal::Accounting::DB::Security->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($securityDBObject->error) { $doc->print($securityDBObject->errorMessage);  return $doc;  }
  #get all lots for account, sub account pair sent in
  my @lotNodes = $securityDBObject->getLotBalanceEntriesFor(accountNum => $account, subAccountNum => $subAccount, ledgerCode => $self->{input}->{ledgerCode}, order => "lot", eventDate => $date);
  if ($securityDBObject->error())
  {
    $doc->print("<font color=\"red\">SecurityDBObject had error</font><br>\n");
    $doc->print( $securityDBObject->errorMessage());
    return $doc;
  }
  if (not defined @lotNodes)
  {#there were no lots or there was a problem
    # lookup the account and sub account names.
    my $accountObj = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
    $accountObj->set("id", $account);
    $accountObj->select();
    if ($accountObj->error)
    {
      $doc->print($accountObj->errorMessage);
      return $doc;
    }
    my $accountName = $accountObj->get("name");
    my $stockObj = Portal::Accounting::Objects::Stock->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
    $stockObj->set("id",$subAccount);
    $stockObj->select;
    if ($stockObj->error)
    {
      $doc->print($stockObj->errorMessage);
      return $doc;
    }
    my $subAccountName = $stockObj->get("name");
    $doc->print("<p><font color=\"red\">There were no lots found for this Account ($accountName) and Security ($subAccountName) and Ledger ($self->{input}->{ledgerCode}) and Date ($date).</font></p>");
    $doc->print("<form>\n");
    $doc->print("<input type=\"button\" value=\"Close\" onClick=\"self.close();\">\n");
    $doc->print("</form>\n");
    return $doc;
  }
  $doc->print(<<"ENDOF");
<p>Enter the Number of Shares to pull from each lot.</p>
<form name="mainform">\n
  <table width="100%">\n
  <tr>\n
    <th align="left">Lot&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th align="left">Date&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th align="right">Shares Avail</th>
    <th>&nbsp;</th>
    <th align="right">&nbsp;&nbsp;&nbsp;&nbsp;Cost</th>
    <th>&nbsp;</th>\n
    <th align="right">&nbsp;&nbsp;&nbsp;&nbsp;Cost/Share</th>
    <th>&nbsp;&nbsp;</th>
    <th align="left">Shares</th>
    </tr>\n
ENDOF
  my $costPerShare;
  my $totalShares = 0;
  my $sharesTotalCost = 0;
  foreach my $index (@lotNodes)
  {
    if ($index->{shares} != 0.0) #dont print this row if the share balance is 0
    {
      $costPerShare = $index->{cost} / $index->{shares}; #/
      $doc->setFocus("body");
      $doc->print(<<"ENDOF");
    <tr>
      <td align="left">$index->{lot}</td>
      <td align="left">$index->{eventDate}</td>
      <td align="right">$index->{shares}</td>
      <td>&nbsp;</td>
      <td align="right">$index->{cost}</td>
      <td>&nbsp;</td>
      <td align="right">$costPerShare</td>
      <td>&nbsp;</td>
      <td><input type="text" name="l$index->{lot}" size="10" value="0.0" onChange="checkMax(this); updateTotal();"></td>
    </tr>
ENDOF
      $doc->setFocus("javascript");
      $doc->print("shareArray[\"$index->{lot}\"]=$index->{shares};\n");
      $totalShares += $index->{shares};
      $sharesTotalCost += $index->{cost};
    }
  }
  $doc->setFocus("body");
  $doc->print(<<"ENDOF");
    <tr>
      <th colspan="2" align="right">Total</th>
      <td align="right"><b>$totalShares</b></th>
      <td>&nbsp;</th>
      <td align="right"><b>$sharesTotalCost</b></th>
      <td>&nbsp;</td>
      <td>&nbsp;</td>
      <td>&nbsp;</td>
      <td><input type="text" name="total" size="10" onFocus="this.blur();"></td>
    </tr>
  </table>
  <table width="100%">
    <tr>
      <td>
        <input type="button" value="All" onClick="selectAll();">
      </td>
      <td>
       <input type="button" value="Done" onClick="writeLots();self.close();">
      </td>
    </tr>
  </table>
</form>
ENDOF
  $doc->setFocus("javascript");
  my $form = "mainForm";
  if (defined $self->{input}->{targetForm}) { $form = $self->{input}->{targetForm};  }
  $doc->print(<<"ENDOF");
function writeLots()
{
  //write data back into parent window into mainForm.lots
  var lotstring ='';
  var total = 0.0;
  var count = 0;
  for (var i in shareArray)
  {
    var val = 0.0;
    val = eval("document.mainForm.l"+i+".value");
    if (val != '' && val > 0)
    {
      total = eval(total + ' + ' + val + ';');
      if (count > 0)
      {
        lotstring += '|';
      }
      lotstring += i + '=' + val;
      count++; # increment count only for valid entries
    }
  }
  total = floatify(total, 6);
  var windowref = window.opener;
  windowref.document.$form.lots.value = lotstring;
  windowref.document.$form.shares.value = total;
}

function checkMax(textfield)
{
  textfield.value = floatify(textfield.value, 6);
  var name = textfield.name;
  name = name.substr(1);
  if (textfield.value > shareArray[name])
  {
    alert("Shares chosen cannot be more than what is available!");
    textfield.value = floatify(shareArray[name], 6);
  }
  if (textfield.value < 0)
  {
    alert("Shares chosen cannot be less than 0!
    textfield.value = '0.0';
    textfield.value = floatify(textfield.value, 6);
  }
}

function updateTotal()
{
  var total = 0.0;
  var val = 0.0;
  var evalme;
  var evalstring;
  for (var i in shareArray)
  {
    evalme = "document.mainForm.l"+i+".value;";
    total += parseFloat(eval(evalme));
  }
  total = floatify(total, 6);
  document.mainForm.total.value = total;
}

function selectAll()
{
  for (var i in shareArray)
  {
    eval("document.mainForm.l" +i+ ".value = shareArray[\"+i+"\"]");
  }
}
ENDOF
  return $doc;
}

sub prepDoc
{
  my $self = shift;
  my $doc = $self->SUPER::prepDoc();
  $doc->setJavascriptInclude(file => $self->{jsDir}."Investments.js");
  return $doc;
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::BaseL(3)

=cut
