# JournalEntry.pm - The Object Class that provides a JournalEntry Object
#$Id: JournalEntry.pm,v 1.27 2005/08/16 22:05:21 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by Xperience, Inc.

package Portal::Accounting::JournalEntry;
use strict;
use Portal::Accounting::Objects::DoubleEntry;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::Search;
use Portal::Accounting::BaseL;
use Portal::Accounting::Objects::Ledger;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

use Date::Calc qw(Today);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.27 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "General Ledger",
  demo => "Demo Data",
  delete => "Delete",
  check => "Check",
  deposit => "Deposit",
  doubleEntry => "Double Entry",
  transfer => "Transfer",
  select => "Recall",
  edit => "Edit",
  insert => "Insert",
  searchForm => "Search Journals",
  search => "Search Journals",
  );

=head1 NAME

JournalEntry

=head1 SYNOPSIS

  use Portal::Accounting::JournalEntry;
  my $obj = Portal::Accounting::JournalEntry->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This set of screens will hold the forms for data Entry in the system.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion, #!!!portal args??)

 Creates a new instance of the Portal::Accounting::JournalEntry module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the JournalEntry State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  $self->{commands} = \%commands;

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->Portal::Accounting::JournalEntry::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{Journals} = {
    G => "doubleEntry",
    CP => "check",
    CSR => "deposit",
    TX => "transfer",
  };
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut

sub c_display
{
  my $self = shift;
  my %args = ( command => "display", message => "", @_ );
  my $command = $args{command};
  my $siteDir = $self->{configObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $message = $args{message};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};

  my $doc = $self->prepDoc;
  my $url = $self->{baseURL};

  my $demo = $self->langObj->map("Demo Data");
  my $Check = $self->langObj->map("Check");
  my $Deposit = $self->langObj->map("Deposit");
  my $Double = $self->langObj->map("Double Entry");
  my $Credit = $self->langObj->map("Credit Purchase");
  my $CreditR = $self->langObj->map("Credit Receipt");
  my $Search = $self->langObj->map("Search");

  my $s = $self->{input}->{state};
  my $a = $self->{input}->{app};
  my $l = $self->{input}->{ledgerCode};
  $doc->print(<<"EOFO");
<a href="?app=$a&state=$s&command=searchForm&ledgerCode=$l">$Search</a><br>
<a href="?app=$a&state=$s&command=check&ledgerCode=$l">$Check</a><br>
<a href="?app=$a&state=$s&command=deposit&ledgerCode=$l">$Deposit</a><br>
<a href="?app=$a&state=$s&command=doubleEntry&ledgerCode=$l">$Double</a><br>
<a href="?app=$a&state=$s&command=creditPurchase&ledgerCode=$l">$Credit</a><br>
<a href="?app=$a&state=$s&command=creditReciept&ledgerCode=$l">$CreditR</a><br>
<a href="?app=$a&state=$s&command=demo&ledgerCode=$l">$demo</a><br>
EOFO
  return $doc;
}

sub c_doubleEntry
{
  my $self = shift;
  my %args = ( message => "", id => -1, @_ );
  my $doc = $self->prepDoc();
  my $lo = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle);
  $lo->set('code', $self->{input}->{ledgerCode});
  $lo->select();
  if ($lo->error) { $self->error("<p>Could not create ledger!</p>\n".$lo->errorMessage); return undef; }
  my $je = Portal::Accounting::Objects::DoubleEntry->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle, ledger => $lo);
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  $je->set('id', $args{id});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  else
  {
    $doc->setFocus("body");
    my %form = $je->printForm(form => "main", command => "display");
    if ($je->error)
    {
      $doc->print("Error<br>".$je->errorMessage);
    }
    $doc->print(%form);
  }

  return $doc;
}

#select is to retrieve a javascript version of a journal entry to be used by the data entry screen
sub c_select
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $lo = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle);
  $lo->set('code', $self->{input}->{ledgerCode});
  $lo->select();
  if ($lo->error) { $self->error("<p>Could not create ledger!</p>\n".$lo->errorMessage); return undef; }
  my $j = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledger => $lo);
  $j->set('id', $self->{input}->{id});
  $doc->setFocus("body");
  if ($j->select() == 0)
  {
    $j->set($j->{id}, -1);
  }
  if ($j->error) { $doc->print($j->errorMessage); }
  else
  {
    if ($j->warning) { $doc->print($j->warningString); }
    $doc->print($j->printHTML);
    $doc->setFocus("javascript");
    $doc->print($j->javascript());
  }
  $doc->setFocus("body");
  $doc->print($j->debugString);
  return $doc;
}

sub c_insert
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $subA = Portal::Accounting::Objects::Account->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  $doc->setFocus("body");
  if ($subA->error)
  {
    $self->error("<p>Could not create Objects::SubAccount in c_insert</p>\n");
    return undef;
  }
  my $success = '-1';
  my $lo = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle);
  $lo->set('code', $self->{input}->{ledgerCode});
  $lo->select();
  if ($lo->error) { $self->error("<p>Could not create ledger!</p>\n".$lo->errorMessage); return undef; }
  my $j = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledger => $lo);
  if ($j->error)
  {
    $doc->print('new failed'.$j->errorMessage);
    return $doc;
  }
  $j->populate(%{$self->{input}});
  $doc->print($j->debugString);
  if ($j->error)
  {
    $doc->print('populate failed'.$j->errorMessage);
  }
  else
  {
    $j->write();
    if ($j->error)
    {
      $doc->print('database write failed'.$j->errorMessage);
    }
    else
    {
      $doc->print($j->printHTML);
      $success = $j->get("id");
    }
  }
  if ($j->warning) { $doc->print($j->warningString); }
  $doc->print($j->debugString);
  my $js = $subA->bankOverhead($lo->get("code"));
  $doc->print($js);
  $doc->setFocus("javascript");
  $doc->print($js);
  $doc->print(<<"ENDOF");
  var js = '$js';
  if (opener)
  {
    if (opener.postInsert) opener.postInsert($success);
    else { alert("function postInsert not found"); }
    if (opener.banks) opener.banks(js);
    else { alert("function banks not found"); }
  }
ENDOF
  return $doc;
}

sub c_demo
{
  my $self = shift;
  my %args = ( message => "", @_ );
  my $phrases = $self->{langObj};
  my $lc = $self->{input}->{ledgerCode};
  my $year = 2004;
  my $doc = $self->prepDoc();
  $doc->setFocus("body");
  if (!$self->{userObj}->{admin})
  {
    $doc->print("Only an administrator can enter demo data!<br>\n");
    return $doc;
  }
  my $sql = "SELECT value FROM config_tb WHERE name = 'demo'";
  my $sth = $self->dbHandle->read($sql);
  my $demo = 0;
  my $P;
  my $E;
  my $W;
  my $I;
  if (my @info = $sth->fetchrow_array) { $demo = $info[0]; }
  if ($demo ne '1')
  {
    $doc->print("<p>Demo is disabled!  You must set demo = 1 in the config_tb.</p>\n");
    return $doc;
  }
  if ($self->{input}->{confirmDemo} ne "1")
  {
    $doc->print(<<"ENDOF");
 Click this button if you are sure that you want to enter demo data into these books.<br>
  <form>
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="demo">
    <input type="hidden" name="ledgerCode" value="$lc">
    <input type="hidden" name="confirmDemo" value="1">
    <input type="submit" name="demo" value="X">
  </form>

<form name="main">
  <input type="hidden" name="command" value="edit" />
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="JournalEntry" />
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
  Enter an id to edit a transaction:
  <input type="text" name="id" value="1" />
  <input type="submit">
</form>
ENDOF
    return $doc;
  }
  my $bankSql = "SELECT (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.num = 0) AS bank";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 5) AS expense";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 4) AS revenue";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 2) AS liability";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 3) AS equity";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.num = 3) AS investment";
  $bankSql .= ", (SELECT working_year('$lc')) AS year";
  $bankSql .= "  FROM account_tb WHERE type = 0";
  my $result = $self->{dbHandle}->read($bankSql);
  if ($self->{dbHandle}->error)
  {
    $doc->print($self->{dbHandle}->errorMessage);
    return $doc;
  }
  my $info;
  if (not $info = $result->fetchrow_hashref)
  {
    $doc->print($self->{dbHandle}->errorMessage);
    return $doc;
  }
  $year = $info->{year};
  if ($year !~ /^\d{4}$/) { my @year = Today; $year = $year[0];}
  my $expense = $info->{expense};
  my $revenue = $info->{revenue};
  my $bank = $info->{bank};
  my $investment = $info->{investment};
  my $equity = $info->{equity};
  my $liability = $info->{liability};
#setup objects
  $doc->print(<<"ENDOF");
<p>using accounts:<br>
bank = $info->{bank}<br>
investment = $info->{investment}<br>
liability = $info->{liability}<br>
equity = $info->{equity}<br>
expense = $info->{expense}<br>
revenue = $info->{revenue}<br>
</p><hr>
ENDOF
  #loop 12 times for inserting 12 months.  start with 2 bc 1 is already set
  my $lo = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->{dbHandle});
  $lo->set('code', $lc);
  $lo->select();
  if ($lo->error) { $doc->print($lo->errorMessage); return $doc; }
  for (my $month = 1; $month < 13; $month++)
  {
    my $elec = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledger => $lo, ledgerCode => $lc);
    my $water = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledger => $lo, ledgerCode => $lc);
    my $pay = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledger => $lo, ledgerCode => $lc);
    my $inv = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledger => $lo, ledgerCode => $lc);
  #populate objects
    $elec->set('payee', 'Some Electric Co');
    $elec->set('remember', 'false');
    $elec->set('date',"$year-$month-10");
    $elec->addRow(counter => 0,
      account => $expense,
      amount => "50.00",
      dorc => "true",
      comment => "a month $month",
    );
    if ($elec->error) { $doc->print($elec->errorMessage); }
    $elec->addRow(counter => 1,
      account => $bank,
      dorc => "false",
      amount => "50.00",
      comment => "b month $month",
    );
    if ($elec->error) { $doc->print($elec->errorMessage); }
    if (! $elec->isValid)
    {
      $doc->print($elec->errorMessage);
      $doc->print("l:".$lc);
#       $doc->print("<br>0: ".$elec->{rowObjects}[0]->get('dorc')."<br>1: ".$elec->{rowObjects}[1]->get('dorc')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('amount')."<br>1: ".$elec->{rowObjects}[1]->get('amount')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('d_amount')."<br>1: ".$elec->{rowObjects}[1]->get('d_amount')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('c_amount')."<br>1: ".$elec->{rowObjects}[1]->get('c_amount')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('account')."<br>1: ".$elec->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    #water object
    $water->set('payee', 'My Water Co');
    $water->set('date', "$year-$month-15");
    $water->addRow(counter => 0,
      comment => "month $month",
      dorc => "true",
      account => $expense,
      amount => "10.00",
    );
    $water->addRow(counter => 1,
      comment => "month $month",
      account => $bank,
      dorc => "false",
      amount => "10.00",
    );
    if (! $water->isValid)
    {
      $doc->print($water->errorMessage);
      $doc->print("<br>0: ".$water->{rowObjects}[0]->dorc."<br>1: ".$water->{rowObjects}[1]->get('dorc')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('amount')."<br>1: ".$water->{rowObjects}[1]->get('amount')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('d_amount')."<br>1: ".$water->{rowObjects}[1]->get('d_amount')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('c_amount')."<br>1: ".$water->{rowObjects}[1]->get('c_amount')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('account')."<br>1: ".$water->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    #pay object
    $pay->set('payee', 'Revenue');
    $pay->set('remember', 'false');
    $pay->set('date', "$year-$month-1");
    $pay->addRow(counter => 0,
      comment => "month $month",
      account => $revenue,
      amount => "1000.00",
      dorc => "false",
    );
    $pay->addRow(counter => 1,
      comment => "month $month",
      account => $bank,
      dorc => "true",
      amount => "1000.00",
    );
    if (! $pay->isValid)
    {
      $doc->print($pay->errorMessage);
      $doc->print("<br>0: ".$pay->{rowObjects}[0]->dorc."<br>1: ".$pay->{rowObjects}[1]->get('dorc')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('amount')."<br>1: ".$pay->{rowObjects}[1]->get('amount')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('d_amount')."<br>1: ".$pay->{rowObjects}[1]->get('d_amount')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('c_amount')."<br>1: ".$pay->{rowObjects}[1]->get('c_amount')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('account')."<br>1: ".$pay->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    #invest object
    $inv->set('payee' , 'Dividend Revenue'    );
    $inv->set('remember' , 'false'    );
    $inv->set('date' , "$year-$month-1"    );
    $inv->addRow(counter => 0,
      comment => "month $month",
      account => $revenue,
      dorc => "false",
      amount => "200.00",
    );
    if ($inv->error) { $doc->print($inv->errorMessage); return $doc; }
    $inv->addRow(counter => 1,
      comment => "month $month",
      account => $investment,
      amount => "200.00",
      dorc => "true",
    );
    if (! $inv->isValid || $inv->error)
    {
      $doc->print($inv->errorMessage);
      $doc->print("<br>0: ".$inv->{rowObjects}[0]->get('dorc')."<br>1: ".$inv->{rowObjects}[1]->get('dorc')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('amount')."<br>1: ".$inv->{rowObjects}[1]->get('amount')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('d_amount')."<br>1: ".$inv->{rowObjects}[1]->get('d_amount')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('c_amount')."<br>1: ".$inv->{rowObjects}[1]->get('c_amount')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('account')."<br>1: ".$inv->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    my $ref;
    my $r;

    #insert current data into DB first
    $ref = $elec->insert(commit => 1);
    if ($elec->error)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$elec->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$elec->get("date")."<br>");
    $ref = $water->insert(commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$water->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$water->get("date")."<br>");
    $ref = $pay->insert(commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$pay->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$pay->get("date")."<br>");
    $ref = $inv->insert(commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$inv->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$inv->get("date")."<br>");
    $W = $water;
    $P = $pay;
    $E = $elec;
    $I = $inv;
  }
  $doc->print("Data Entered successfully for $year<br>\n");
  $doc->print('<hr>');
  $doc->print($W->printHTML);
  $doc->print('<hr>');
  $doc->print($E->printForm);
  $doc->print('<hr>');
  $doc->print($P->printXML);
  $doc->print('<hr><textarea rows="8" cols="80">');
  $doc->print($I->javascript);
  $doc->print('</textarea><hr>');

  $doc->print(<<"ENDOF");
<form name="main">
  <input type="hidden" name="command" value="edit" />
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="JournalEntry" />
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
  Enter an id to edit a transaction:
  <input type="text" name="id" value="1" />
  <input type="submit">
</form>
ENDOF
  return $doc;
}

sub c_edit
{
  my $self = shift;
  my $sql = "SELECT journal_code AS journal FROM journal_entry_tb WHERE ref_num = '$self->{input}->{id}'";
  #!!!get Journal Code for given id and then create object of correct type or call correct subroutien
  my $sth = $self->{dbHandle}->read($sql);
  my $name = "doubleEntry";
  if (my @code = $sth->fetchrow_array)
  {
    $name = $self->{Journals}->{$code[0]};
  }
  else { die("Could not get journal code in JournalEntry->c_edit:$sql"); }
  eval('return $self->c_'.$name.'(id => $self->{input}->{id}, @_);');
}

sub c_delete
{
  my $self = shift;
  my $doc = $self->prepDoc(@_);
  if ($self->{input}->{id} !~ /^\d+$/)
  {
    $doc->print("<p>Invalid transaction id sent to delete: $self->{input}->{id}</p>\n");
  }
  else
  {
    #create new journal entry
    my $result;
    my $l = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
    $l->set('code', $self->{input}->{ledgerCode});
    $l->select;
    if ($l->error) { $self->error("<p>Could not retrieve ledger object from database for code $self->{input}->{ledgerCode}!</p>\n"); return undef; }
    my $o = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledger => $l, @_);
    if (!defined $o) { $doc->print("<p>Could not create object JournalEntry.</p>\n");  }
    if ($o->error) { $doc->print("<p>Could not use object: ".$o->errorMessage."</p>\n");  }
    $o->set('id', $self->{input}->{id});
    if (! $o->error)  {  $result = $o->delete;  }
    if ($o->error) { $doc->print($o->errorMessage); }
    else
    {
      if ($result == -1)
      {
        $doc->print('Delete failed<hr>'.$o->errorMessage);
      }
      else
      {
        $doc->print('Successfully deleted '.$result); #!!!add code to write back to parent window
      }
    }
  }
  return $doc;
}

sub c_searchForm
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my %args = ( command => $self->{input}->{command}, @_ );
  my $command = $args{command};
  my $l = $self->{langObj};
  my $foo = Portal::Accounting::Objects::Search->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode}, @_);
  if ($foo->error) { $doc->print($foo->errorMessage); return $doc;  }
  if ($command eq "search")
  { #validate user input then react
    if ($foo->populate(%{$self->{input}}) && $foo->validateForm(input => $self->{input}, formName => "main"))
    {
      $self->{OBJECT} = $foo;
      my @je = $self->selectArray();
      if ($self->error)
      {
        $doc->print($self->errorMessage);
        $self->resetError;
      }
      else
      {
       if ($self->warning) { $doc->print($self->warningString); }
       $doc->print($self->debugString);
       # then display them by walking and $o->printHTML
        if (scalar @je == 0) { $doc->print($l->map("Found")." 0"); }
        else { $doc->print($je[0]->printHeaders); }
        foreach my $bar (@je)
        {
          $bar->isValid;
          $doc->print($bar->printForm);
          $doc->print($bar->errorMessage);
        }
        return $doc;
      }
    }
  }
  #display form
  my %form = $foo->printForm();
  if ($foo->error)
  {
    $self->error($foo->errorMessage);
  }
  if (defined %form)
  {
    $doc->print(%form);
  }
  else
  {
    $doc->print('error! form is undefined<br>');
    $doc->print("\nformObj:".$foo->{formObj}->error);
  }

  $doc->print($foo->debugString);
  return $doc;
}

sub c_search
{
  my $self = shift;
  return $self->c_searchForm(@_);
}

sub selectArray
{
  my $self = shift;
  my $return;
  my $l = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  $l->set('code', $self->{input}->{ledgerCode});
  $l->select;
  if ($l->error) { $self->error("<p>Could not retrieve ledger object from database for code $self->{input}->{ledgerCode}!</p>\n"); return undef; }
  my $sql = $self->{OBJECT}->selectSQL();
#  $self->error($sql.$self->{OBJECT}->{debugString});
  $self->debug("<br>".$sql.$self->{OBJECT}->{debugString});
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef;  }
  while (my @d = $result->fetchrow_array)
  {
    my $o = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledger => $l, @_);
    if (!defined $o) { $self->warning("<p>Could not create object JournalEntry.</p>\n");  next;  }
    if ($o->error) { $self->warning("<p>Could not use object: ".$o->errorMessage."</p>\n");  next;  }
    $o->set('id', $d[0]);
    if (! $o->error)  {  $o->select;  }
    if ($o->error) { $self->warning($o->errorMessage);  next; }
    push @{$return}, $o;
  }
  if (not defined $return) { return (); }
  return @{$return};
}

sub prepDoc
{
  my $self = shift;
  my $doc = $self->SUPER::prepDoc(@_);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Journal.js");
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
