# Ledger.pm
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Ledger.pm,v 1.91 2005/08/16 21:45:21 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

Ledger - Object used to output Ledger screens

=head1 SYNOPSIS

  use Portal::Accounting::Ledger;
  my $obj = Portal::Accounting::Ledger->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Ledger is an accounting ledger class.

=cut
package Portal::Accounting::Ledger;
my $state = "Ledger";
use strict;
use Portal::Accounting::DB::Main;
use Portal::Accounting::BaseL;
use Portal::Accounting::DB::Ledger;
use Portal::Accounting::Objects::CloseOut;
use Portal::Accounting::Objects::OpenBooks;
use Portal::Accounting::Investments::WatchList;
use HTMLObject::Form;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.91 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
              display => "Ledger Summary",            #always checks for output variable
              closeLedger => "Close Ledger",
              'open' => "Open",
              ledgerFrame => "Ledger Frame",
              ledgerMenu  => "Leger Menu",
              displayCloseOut => "Close Books",
              closeOut => "Close Books",
              displayReOpen => "Open Books",
              reOpen => "Open Books",
              );

=head1 Exported FUNCTIONS


B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(langObj)

 Creates a new instance of the Portal::Accounting::Ledger module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self;
  eval { $self = $class->SUPER::new(@_); };
  if ($@)
  {
    die $@;
    return $self;
  }
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # do validation
  if (!$self->Portal::Accounting::Ledger::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;
  $self->{'DB::Ledger'} = Portal::Accounting::DB::Ledger->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj}) ;
  return $self;
}

=head2 bool isValid(void)

    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # do validation code here.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub c_display
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $errStr = "Portal::Accounting::Ledger->display()  - Error!<br>\n";
  my $siteDir = $self->{oldCGIDir};
  my $portalCGIDir = $self->{portalCGIDir};
  my $phrases = $self->{langObj};
  my $currVersion  = $self->{release} . "." . $self->{DBVersion};
  my $mainHeight = "600";
  my $mainWidth = "1024";

  my $url = $self->{baseURL};
  my $mainDB = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $phrases);
  if ($mainDB->error)
  {
    $self->prefixError("Could not initialize DB::Main object!<br>".$mainDB->errorMessage."\n");
    return undef;
  }
  my $doc = $self->prepDoc;
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $color = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{color};
  my $ledger = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name};

  my $portfolioPhrase = $phrases->map("Portfolio");

  # generate javascript functions needed.
  $self->{methods}->includeCoreJSFiles(doc => $doc);
  if ($self->{methods}->error)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  $doc->setFocus("body");
  $doc->print(<<"ENDOF");
  <table border="0" width="100%">
    <tr>
      <td width="15%" valign="top">
ENDOF
  $doc->print("        <a href=\"?$self->{getInfo}&command=displayCloseOut\">".$phrases->map("Close Books")."</a><br>");
  $doc->print("        <a href=\"?$self->{getInfo}&command=displayReOpen\">".$phrases->map("Open Books")."</a><br>");
  $doc->print("        <a href=\"?$self->{getInfo}&command=initialBalances\">".$phrases->map("Initial Balances")."</a><br>");
  $doc->print("        <a href=\"\" onclick=\"javascript: { windowName = getTimeStamp(); window.open('$url?app=Accounting&state=Main&windowName='+windowName+'&command=edit&ledgerCode=$ledgerCode&ledgerOpen=1', windowName , 'width=$mainWidth,height=$mainHeight,scrollbars,status,toolbar'); return false; }\">".$phrases->map("Edit")." ".$phrases->map("Ledger")."</a>");
  $doc->print(<<"ENDOF");
      </td>
      <td valign="top">
        <center>
        <style type="text/css">
        <!--
          .CashSummaryTitle { font-weight: bold; text-decoration: none; }
          .CashSummaryTable { }
          .CashSummaryName { text-transform: capitalize; }
          .CashSummaryBalance {text-align: right; }
          .CashSummaryReconcile { }
          .CashSummaryLastEntry { }
          .negative { color: #ff0000; }
        -->
        </style>
ENDOF
  my $html = $mainDB->getCashAccountSummary(ledgerCode => $self->{input}->{ledgerCode});
  if ($mainDB->error)
  {
    $html = $mainDB->errorMessage;
  }
  if (not defined $html)
  {
    $html = "Error during Cash account Summary!<br>" . $mainDB->errorMessage();
  }
  $doc->print($html);
  $doc->print(<<"ENDOF");
      </td>
    </tr>
  </table>
  <hr noshade size="2">
  <iframe name="watchList" align="top" frameborder="0" width="100%" scrolling="yes" Height="600" src="?app=$self->{app}&state=Investments::WatchList&command=view&ledgerCode=$ledgerCode&groupName=SHORT"></iframe>
ENDOF
  return $doc;
}

sub c_closeLedger
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my $errStr = "Portal::Accounting::Ledger->closeLedger()  - Error!<br>\n";
  my $siteDir = $self->{portalConfigObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $windowName = $self->{input}->{windowName};

  # remove this ledger from the window list.
  if ($self->{sessionObj}->windowExists($windowName))
  {
    $self->{sessionObj}->unregisterWindow($windowName);
  }
  if ($self->{sessionObj}->error)
  {
    $self->error($self->{sessionObj}->errorMessage);
    return undef;
  }

  # generate the document to close this window.
  my $onLoadCode = "window.close();\n";
  $doc->setOnload(code => $onLoadCode);
  $doc->setTitle($phrases->map("Close Window")." - $windowName");
  $doc->setFocus("body");
  $doc->print($phrases->map("Close Window")." - $windowName\n");

  return $doc;
}

sub c_ledgerMenu
{
  my $self = shift;
  my %args = (@_ );
  my $doc = $self->prepDoc();
  my $errStr = "Portal::Accounting::Main->ledgerMenu()  - Error!<br>\n";
  my $phrases = $self->{langObj};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $ledger = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name};
  my $target = "";
  my $windowName;
  my $mainHeight = "600";
  my $mainWidth = "1024";

  my $url = $self->{baseURL};
  my $jsUrl = $self->{jsDir};

  if (exists $self->{input}->{windowName})
  {  $windowName = $self->{input}->{windowName}; }
 if ($windowName eq "")
 {
   $self->setError( errorString => "Did not receive a windowName for parent of frameset<br>\n");
   return undef;
 }
  $target = $windowName . "Data";
  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");
function getTimeStamp()
{
  var timeStamp = new Date();
  return timeStamp.getTime();
}

function openLedger(windowName)
{
  var foo = new Array();
  if (windowName == null || windowName == '') { windowName = getTimeStamp(); }
  foo.app = 'Accounting';
  foo.state = 'Ledger';
  foo.command = 'open';
  foo.ledgerCode = '$self->{input}->{ledgerCode}';
  foo.windowName = windowName;
  var options = ' width=$mainWidth,height=$mainHeight,scrollbars,status,toolbar';
  myPost(windowName, '', foo, options);
}
END_OF_CODE

  my $newFrame = $phrases->map("New Frame");
  my $newW = $phrases->map("New Window");
  my $closeW = $phrases->map("Close Window");
  $doc->setBase(target => $target);
  $doc->setFocus("body");
  #  New frameset must use get bc of target=_top.  does not work as post
  #  X close window does not work in tabs.  only the first instance works for some reason.
  $doc->print(<<"END_OF_CODE");
<table width="100%">
  <tr>
    <td align="right">
      <font size="+2">
        <a href="?$self->{getInfo}&command=ledgerFrame" target="_top" title="$newFrame">F</a>
        | <a href="" onclick="javascript: openLedger(); return false;" title="$newW">W</a>
        | <a href="" onclick="myPost('$windowName', '', '$self->{getInfo}&command=closeLedger&ledgerCode=$ledgerCode'); return false;" title="$closeW" target="_top">X</a>
      </font>
    </td>
</table>
<br>
END_OF_CODE
  $doc->print("\n <p> <a href=\"$url?command=display&$self->{getInfo}\">".$phrases->map("Summary")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=Accounts&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Chart of Accounts")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=JournalEntry&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Journals")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=JournalEntry&command=searchForm&ledgerCode=$ledgerCode\">".$phrases->map("Search")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=SubAccounts&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Sub Accounts")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=SubEntities&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Sub Entities")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=Register&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Registers")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=Payroll&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Payroll")."</a>");
  $doc->print("\n <br> <a href=\"$url?app=$self->{app}&state=Investments&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Investments")."</a>");
  $doc->print("\n <br> <a href=\"$self->{jsDir}accounting/cgi-bin/report.cgi?app=$self->{app}&state=Reports&command=display_for_running&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Reports")."</a>");

  return $doc;
}

=item c_open

  This routine is to be called when a new window has been opened for a ledger.
  It will register the window with the Portal then open the requested ledgerFrameSet.

  requires:  windowName, ledgerCode
=cut
sub c_open
{
  my $self = shift;
  $self->{sessionObj}->registerWindow($self->{input}->{windowName});
  if ($self->{sessionObj}->error)
  {
    $self->error($self->{sessionObj}->errorMessage);
    return undef;
  }
  return $self->c_ledgerFrame(@_);
}

sub c_ledgerFrame
{
  my $self = shift;
  my %args = ( @_ );
  my $errStr = "Portal::Accounting::Ledger->ledgerFrame()  - Error!<br>\n";
  my $phrases = $self->{langObj};
  my $framedoc = HTMLObject::FrameSet->new();
  my $windowName;
  my $menuSize = 120; #should come from user prefs#!!!

  my $url = $self->{baseURL};

  if (exists $self->{input}->{windowName})
  {
    $windowName = $self->{input}->{windowName};
    if ($windowName eq "")
    {
      $self->setError(errorString => $errStr . "Window name was not set!<br>\n");
      return undef;
    }
  }
  else
  {
    $self->setError( $self->errorMessage() . "Did not receive a windowName for frameset<br>\n");
    return undef;
  }
  $framedoc->setTitle($self->{sessionObj}->{store}->{ledgers}->{$self->{input}->{ledgerCode}}->{name});

  my $inner_content;
  $inner_content = $framedoc->createNoframe($phrases->map("frameBrowser"));
  $inner_content .= $framedoc->createFrame(
    src => "$url?$self->{getInfo}&command=display",
    frameborder => 'yes',
    scrolling => 'auto',
    name => $windowName . "Data",
    bordercolor => 'white',
    marginwidth => '1');
  $inner_content .= $framedoc->createFrame(
    src => "$url?$self->{getInfo}&command=ledgerMenu",
    frameborder => 'yes', scrolling => 'auto',
    name => $windowName . "Menu",
    bordercolor => 'white',
    marginwidth => '0',
    marginheight => '0',
    border => '1');
  $framedoc->outerFrameset(
    cols => "*, $menuSize",
    name => $windowName,
    content => $inner_content,
    border => '3',
    frameborder => 'yes');
  return $framedoc;
}

sub c_displayReOpen
{
  my $self = shift;
  return $self->c_reOpen;
}

sub c_reOpen
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $formObj = HTMLObject::Form->new(langObj => $self->{langObj});
  my $db = $self->{'DB::Ledger'};
  my $reopen = Portal::Accounting::Objects::OpenBooks->new(langObj => $self->{langObj});
  my $phrases = $self->{langObj};
  my $result = 1;
  my $ledgerCode = $self->{input}->{ledgerCode};

  if ($self->{input}->{command} eq "reOpen")
  {
    $result = $formObj->validate(profile => $reopen->{profile}, input => $self->{input});
    if ($formObj->error)
    {
      $doc->print($formObj->errorMessage);
    }
    if (!$result)
    {
      $doc->setFocus("body");
      $doc->print($formObj->genErrorString("all"));
    }
    else
    {
#      my $ledger = $db->reopenLedger( ledger => $self->{sessionObj}->{store}->{ledgers}->{$self->{input}->{ledgerCode}}, date => $self->{input}->{date});
      my $ledger = $db->reopenLedger( ledgerCode => $self->{input}->{ledgerCode}, date => $self->{input}->{date});
      if (not defined $ledger)
      {
        $doc->print($db->errorMessage);
      }
      else
      {
        $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode} = $ledger;
        if ($db->error)
        {  $self->prefixError($db->errorMessage);  }
        else
        {
          $doc->print($phrases->map("Success") ." ". $phrases->map("Open Books") . " "
          . $self->{input}->{date} . " "
          . $self->{sessionObj}->{store}->{ledgers}->{$self->{input}->{ledgerCode}}->{name}
          );
        }
        return $doc;
      }
    }
  }
  if (! $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->isValid)
  {
    $doc->setFocus("body");
    $doc->print("Warning!  $ledgerCode is invalid.<br>");
  }

  my @dates = $db->getCloseOutDates($ledgerCode);
  if (scalar @dates == 0)
  {
    $doc->print($phrases->map("noDates"));
  }
  else
  {
    $reopen->populate(date => \@dates, ledgerCode => $ledgerCode );
    if ($reopen->error )
    {
      $self->prefixError($reopen->errorMessage);
      return undef;
    }
    my %form = $formObj->generate(
              data => $reopen->{data}, template => $reopen->{template},
              name => "mainForm", action => $self->{baseURL},
              method => "post", profile => $reopen->{profile});
    if ($formObj->error)
    {
      # display the error message
      $self->prefixError( $formObj->errorMessage);
      return undef;
    }
    $form{javascript} .= $reopen->{javascript};
    $doc->print(%form);
  }
  return $doc;
}

sub c_displayCloseOut
{
  my $self = shift;
  return $self->c_closeOut;
}

sub c_closeOut
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $formObj = HTMLObject::Form->new(langObj => $self->{langObj});
  my $db = $self->{'DB::Ledger'};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $closeout = Portal::Accounting::Objects::CloseOut->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledger => $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode});
  my $ledgerName = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name};
  my $phrases = $self->{langObj};
  my $result;

  if ($self->{input}->{command} eq "closeOut")
  {
    $closeout->populate(%{$self->{input}}); #year => $fiscal, businessType => $businessType, lastClose => $lastClose, dates => \@dates, ledgerCode => $ledgerCode, validate => 0 );
    $result = $closeout->validate();
    if (!$closeout->error)
    {
      my $foo;
      my $bar;
      ($foo = $self->{input}->{date}) =~ s/-//g;
      ($bar = $db->getLastClose($ledgerCode))  =~ s/-//g;
      if ($foo <= $bar)
      {
        $doc->print($phrases->map("Error").".  ". sprintf $phrases->map("closeoutDateInvalid"), $phrases->map("Close Out"), $self->{input}->{date});
      }
      else
      {
        my $ledger = $db->closeoutLedger( ledger => $self->{sessionObj}->{store}->{ledgers}->{$self->{input}->{ledgerCode}}, date => $self->{input}->{date}, accounts => $self->{input}->{account});
        if (not defined $ledger)
        {
          $doc->print($db->errorMessage);
        }
        else
        {
          $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode} = $ledger;
          if ($db->error)
          {  $self->prefixError($db->errorMessage);  }
          $doc->print($phrases->map("Success") ." ". $phrases->map("Close Books") . " "
          . $closeout->{data}->{date}->{-Value} . " "
          . $self->{sessionObj}->{store}->{ledgers}->{$self->{input}->{ledgerCode}}->{name}
          );
          return $doc;
        }
      }
    }
  }

  $doc->print($closeout->printForm);
  if ($closeout->error )
  {
    $doc->print($closeout->errorMessage);
  }
  $doc->print($self->errorMessage);
  return $doc;
}

1;
__END__

=head2

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

JT Moree
PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
