# Base.pm - The Object Class that provides a Base Object
#$Id: BaseL.pm,v 1.5 2005/08/08 22:18:37 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::BaseL;
use strict;
use Portal::Accounting::Objects::Base;;

use vars qw($AUTOLOAD $VERSION @ISA @EXPORT %Elements);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.5 $';
%Elements = ();

=head1 NAME

BaseL - Base Object for deriving other objects. Derives from Objects::Base but also checks for ledgerCode.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::BaseL;
  @ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);

=head1 DESCRIPTION

BaseL is a Base class for Accounting Objects such as
Objects::SubEntity.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Base.
    See Portal::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", @_ );
  $self->{name} = "Portal::Accounting::Objects::BaseL";
  if ($args{ledgerCode} eq "") { $self->error("<p>LedgerCode must be specified for this object!</p>\n");  }
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{data}->{main} = {
    ledgerCode => { -Type => "hidden", -DataType => "string", -Value => $args{ledgerCode} }
  };
  # dont do validation because most objects will start as an empty template
  # do anything else you might need to do.
  return $self;
}

=item bool isValid(void)

 Returns 0 for invalid or 1 for valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub printList
{
  my $self = shift;
  return $self->SUPER::printList(ledgerCode => $self->get("ledgerCode"));
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Base(3)

=cut
