#$Id: DoubleEntry.pm,v 1.36 2005/08/16 21:45:22 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

DoubleEntry - Object used to hold forms for DoubleEntry and manipulate a DE transaction

=head1 SYNOPSIS

  use DoubleEntry;
  my $transobj = DoubleEntry->new(langObj =>);
  if ($transobj->error())
  {
    die $transobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $transobj->set("id", "1234");
  my $form = $transobj->printForm();

=head1 DESCRIPTION

DoubleEntry is a class that stores data, methods, and forms for a
General Journal double entry view.  It derives from JournalEntry so it provides methods
to write to the database or update an existing record in the database.
It's main purpose is to generate a Double Entry form for the user to enter data into the system.

=head1 Exported FUNCTIONS

=cut

package Portal::Accounting::Objects::DoubleEntry;

use strict;
use Portal::Accounting::Objects::JournalEntry;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Objects::SubEntity;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::JournalEntry Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.36 $';

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => "", @_);
  my $act = Portal::Accounting::Objects::Account->new(@_);
  my $sact = Portal::Accounting::Objects::SubAccount->new(@_);
  my $se = Portal::Accounting::Objects::SubEntity->new(@_);
  if ($act->error) { $self->error($act->error); }
  if ($sact->error) { $self->error($sact->error); }
  if ($se->error) { $self->error($se->error); }
  my $l = $self->{langObj};
  #use form object to get calculator, date and some non repeating fields
  $self->{data}->{main} = {
    "id" => { -Label => $l->map("Id"), -Type => "hidden", -Value => -1, -DataType => "int" },
    "view" => { -Type => "hidden", -Value => "G", -DataType => "string" },
    "journal" => { -Type => "select", -Value => "G", -DataType => "string",
      -Options =>  $self->{formObj}->createSelectOptions($self->{journals}) },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Journal" },
    "command" => { -Type => "hidden", -Value => "display" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "date" => { -Type => "date-picker", size => "7", -Label => $l->map("Date"),
      -Value => $self->getCurrentDate(),
      -WidgetOptions => { year => $self->{data}->{main}->{workingYear}->{-Value} },
    },
#     "debit_" => { -Type => "calculator", -Label => $l->map("Debit"), class => "amount",
#       -WidgetOptions => { calcButton => 0, undoButton => 0, helpLink => 0 }
#     },
#     "credit_" => { -Type => "calculator", -Label => $l->map("Credit"), class => "amount",
#       -WidgetOptions => { calcButton => 0, undoButton => 0, helpLink => 0 }
#     },
    "payee" => { -Type => "searchBox", -Label => $l->map("Description"),
      -WidgetOptions => {shareData => "PAYEES", isAA => 1, isSorted => 0,onblur => 'recallData();' },
      -onload => 'init(document.main.id.value);',
    },
    "remember" => { -Type => "checkbox", },
  };
  $self->{css}->{main} = <<"ENDOF";
    input, select { font-size: 80%; }
    input[type="button"] { }

    .row { background: #aaaaaa; }
    .heading { background: #cccccc; }
    .account { border: none; width: 10em; }
    .subAccount { border: none; width: 10em; }
    .subEntity { border: none; width: 3em; }
    .amount { border: none; width: 5em; text-align:right; }
    .subEntityDisplay { display: none; }
    .comment { border: none; }
    .payee {}
ENDOF
  my $Account = $l->map("Account");
  my $SubAccount = $l->map("Sub Account");
  my $Debit = $l->map("Debit");
  my $Credit = $l->map("Credit");
  my $Comment = $l->map("Comment");
  my $SubEntity = $l->map("Sub Entity");
  my $SE = $l->map("SE");
  my $Enter = $l->map("Insert"). " ". $l->map("Transaction");
  $self->{template}->{main} = <<"ENDOF";
<div id='G' style="display: none;" class="row">
  <input type="text" id="account_Display" name="account_Display" value="" class="account" onChange="clearMe(this); " />
  <input type="text" id="subAccount_Display" name="subAccount_Display" value="" class="subAccount" onChange="clearMe(this); " />
  <input type="text" id="debit_" name="debit_" value="" class="amount" onChange="calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); debitCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }" />
  <input type="text" id="credit_" name="credit_" value="" class="amount" onChange="calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); creditCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }" />
  <input type="text" id="comment_" name="comment_" value="" class="comment" />
  <input type="text" id="subEntity_Display" name="subEntity_Display" value="" class="subEntity" onChange="clearMe(this);" />

  <input type="hidden" name="cleared_" value="_">
  <input type="button" name="delete_" value="X" onclick="return deleteRow(this);" style="border: none;" />

  <input type="hidden" name="account_">
  <input type="hidden" name="subAccount_">
  <input type="hidden" name="subEntity_">
<!--
  <input type="text" name="account_" size="3" >
  <input type="text"  size="3" name="subAccount_">
  <input type="text"  size="3" name="subEntity_">
-->

</div>
<div id="newDiv" style="display: none;">
  <center><h4 style="background: #2b7e07; color: #ffffff;">#LANG=New#</h4></center>
</div>
<div id="editDiv" style="display: none;" >
  <center><h4 style="background: #8f0606; color: #ffffff;">#LANG=Edit#</h4></center>
</div>
<div id="pleaseDiv" style="display: none;">
  <center><h4>#LANG=Please Wait#</h4></center>
</div>
<div id="mainDiv">
  <form name="main" target="_blank" method="post" action="">
    <table border="0" cellpadding="5" cellspacing="0" width="100%">
      <tbody>
        <tr>
          <td style="vertical-align: top;">
            #FIELD=id#
            #FIELD=app#
            #FIELD=state#
            #FIELD=command#
            #FIELD=ledgerCode#
            #FIELD=journal#
            <a href="" title="#LANG=Remembered#" onClick="javascript: return false;" disabled="1">#LANG=R#</a> #FIELD=remember#
            #LABEL=date# #FIELD=date#
            #LABEL=payee# #FIELD=payee#
            <input type="checkbox" name="recall" title="#LANG=Allow Recall#" checked="1" onClick="javascript: recallCheck(); " />
          </td>
          <td style="vertical-align: top;">
            <input type="button" value="#LANG=Reset#" onclick="return resetForm();" />
          </td>
          <td style="vertical-align: top;">
            <input onclick="javascript: return addRow(this.form.view.value,'insertRowHere');" name="add" value="#LANG=Add Row#" type="button">
          </td>
        </tr>
      </tbody>
    </table>
    <div class="heading">
      <table style="border: 0;"><tr>
      <td class="accountHeading" style="width: 8em;">$Account</td>
      <td class="subAccountHeading" style="width: 8em;">$SubAccount</td>
      <td class="debitHeading" style="width: 4em;">$Debit</td>
      <td class="creditHeading" style="width: 4em;">$Credit</td>
      <td class="commentHeading" style="width: 10em;">$Comment</td>
      <td class="subEntityHeading"><a href="" title="$SubEntity" onClick="return false;" disabled="1">$SE</a> &nbsp;&nbsp;&nbsp;    </td>
      <td><a title="#LANG=Delete#" onclick="return false;" href="">D</a></td>
      </tr></table>
    </div>
    <span id="insertRowHere"></span>
    <div style="background: #dddddd;"><img width="1" height="5"></div>
    <table border="0" width="100%">
      <tr>
        <td align="left">
          <input name="submitDataButton" onclick="return submitWrapper();" value="Insert" type="button">
        </td>
        <td align="right">
          <a title="#LANG=editHelp#">#</a> <input type="text" name="editID" size="4" title="#LANG=editHelp#" /> <input type="button" name="edit" Value="#LANG=Edit#" onclick="document.main.id.value=document.main.editID.value; init(); " title="#LANG=editHelp#" />
        </td>
      </tr>
    </table>
    <p>&nbsp;</p>
    <div id="optionsA" name="optionsA" >
      <input type="button" name="hideA" value="#LANG=Show Options#" onclick="hideDiv('optionsA'); showDiv('optionsB');">
    </div>
    <div id="optionsB" name="optionsB" class="edit" style="display: none;" >
      <input type="button" name="hideB" value="#LANG=Hide#" onclick="hideDiv('optionsB'); showDiv('optionsA');"><br>
      <input type="checkbox" name="confirmDelete" checked="1"> #LANG=Confirm Deletes#?<br>
      #LANG=When creating new rows#:<br>
      #LANG=Show accounts by#
      <select name="accountSwitch">
        <option value="name">Names</option>
        <option value="code">Codes</option>
      </select>
      <br>
      #LANG=Show subaccounts by#
      <select name="subAccountSwitch">
        <option value="name">Names</option>
        <option value="code">Codes</option>
      </select>
    </div>
    #FIELD=view#
  </form>
</div>
<div id="popupDiv" style="display: none;">
  <form name="popupForm">
    <input type="hidden" name="what" />
    <input type="button" name="popupFinished" value="#LANG=Cancel#" onclick="cancel(); myFocus(this.form.what.value); " />
    <br>
    <iframe id="popupFrame" name="popupFrame" src="" style="width: 100%; height: 30em; border: 0; " />
  </form>
</div>
ENDOF
  $self->{javascript}->{main} .= $self->PAYEES(style => 'G', days => 400, journal => 'G') . "\n";
  $self->{javascript}->{main} .= $act->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $sact->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $se->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= <<"ENDOF";
  langObj['acctWarning']      = "#LANG=acctWarning#";
  langObj['Activating']       = "#LANG=Activating#";
  langObj['already exists']   = "#LANG=already exists#";
  langObj['areUsure']         = "#LANG=areUsure#";
  langObj['CannotCreateSubs'] = "#LANG=CannotCreateSubs#";
  langObj['Code']             = "#LANG=Code#";
  langObj['codePrompt']       = "#LANG=codePrompt#";
  langObj['Create SubEntity'] = "#LANG=Create SubEntity#";
  langObj['Delete all rows?'] = "#LANG=Delete all rows?#";
  langObj['Delete row']       = "#LANG=Delete row#";
  langObj['for this session'] = "#LANG=for this session#";
  langObj['Insert Failed']    = "#LANG=Insert Failed#";
  langObj['Insert']           = "#LANG=Insert#";
  langObj['Invalid']          = "#LANG=Invalid#";
  langObj['Invalid Account']  = "#LANG=Invalid Account#";
  langObj['Invalid Date']     = "#LANG=Invalid Date#";
  langObj['Invalid Page']     = "#LANG=Invalid Page#";
  langObj['is inactive']      = "#LANG=is inactive#";
  langObj['MetaPrompt']       = "#LANG=MetaPrompt#";
  langObj['minimumRows']      = "#LANG=minimumRows#";
  langObj['Name']             = "#LANG=Name#";
  langObj['namePrompt']       = "#LANG=namePrompt#";
  langObj['New Account']      = "#LANG=New Account#";
  langObj['New Sub Account']  = "#LANG=New Sub Account#";
  langObj['noTransaction']    = "#LANG=noTransaction#";
  langObj['notExist']         = "#LANG=notExist#";
  langObj['numbers only']     = "#LANG=numbers only#";
  langObj['reconciledRow']    = "#LANG=reconciledRow#";
  langObj['rightClick']       = "#LANG=rightClick#";
  langObj['row']              = "#LANG=row#";
  langObj['rowWarning']       = "#LANG=rowWarning#";
  langObj['Stop checking?']   = "#LANG=Stop checking?#";
  langObj['subAccountAssignWarning'] = "#LANG=subAccountAssignWarning#";
  langObj['Success']          = "#LANG=Success#!";
  langObj['taxPrompt']        = "#LANG=taxPrompt#";
  langObj['Taxable']          = "#LANG=Taxable#";
  langObj['Type']             = "#LANG=Type#";
  langObj['typePrompt']       = "#LANG=typePrompt#";
  langObj['Update']           = "#LANG=Update#";

function submitWrapper()
{
  if (checkPage()) { return submitData(); }
  alert(langObj['Invalid Page']);
  return false;
}

ENDOF
  if ($act->error) { $self->error($act->errorMessage); }
  if ($sact->error) { $self->error($sact->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }

  $self->{profile}->{main} = {
    required => [  ],
    optional => [  ],
    constraints => {
    },
  };
  $self->{defaultForm} = "main";

  return $self;
}

=item %hash printForm()

=cut
sub printForm
{
   my $self = shift;
   my %form = $self->SUPER::printForm(@_);
   $form{javascript} = $self->langSubstitute($self->{javascript}->{main});
   $form{css} = $self->{css}->{main};
   return %form;
}

1;
__END__

=pod

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

JT Moree

=head1 SEE ALSO

perl(1), JournalEntry(3), DBIWrapper(3)

=cut
