# StockSplit.pm - The Object Class that provides a StockSplit Object
#$Id: StockSplit.pm,v 1.3 2005/07/22 18:51:36 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Investments::StockSplit;
use strict;
use Portal::Accounting::Objects::Investment;
use Portal::Accounting::Objects::ChartOfAccounts;
use Portal::Accounting::Objects::Stock;
use Portal::Accounting::Objects::SubEntity;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Investment Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.3 $';

=head1 NAME

StockSplit - Object used to create, edit, save an StockSplit.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::StockSplit;
  my $obj = Portal::Accounting::Objects::StockSplit->new();
  $obj->populate(
    name => "test", type => 1, code => "1234", );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

StockSplit is a StockSplit class.  This represents a sub account in the
accounting system.  Right now number is used internally. Name and code
are user defined.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::StockSplit module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "splt";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  my $Shares = $l->map("youWillHaveTotal");
  my $Shares2 = $l->map("sharesForEvery");
  my $that = $l->map("thatYouOwn");
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", -Value => "", -DataType => "int" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $self->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -WidgetOptions => { year => $date[0] }, -Value => $today },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks}      },
    "totalShares" => { -Type => "text", -DataType => "float", -Label => $Shares, size => 8 },
    "oldShares" => { -Type => "text", -DataType => "float", -Label => $Shares2, size => 8},
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $Submit = $l->map("Submit");
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=id#
  #FIELD=ledgerCode#
  <h4>#LABEL=type##FIELD=type#</h4>
  <table class="main">
    <tr>
      <td>
        #LABEL=date# #FIELD=date# #LABEL=stock# #FIELD=stock#<br>
        #LABEL=totalShares# #FIELD=totalShares#
        #LABEL=oldShares# #FIELD=oldShares#
        $that.
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{StockSplit} = {
    required => [ qw( date oldShares totalShares stock ) ],
    optional => [ ],
    constraints => {
         totalShares => "float",
         oldShares => "float",
         stock => 'integer',
       },
  };

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
