# StockTransfer.pm - The Object Class that provides a StockTransfer Object
#$Id: StockTransfer.pm,v 1.3 2005/07/22 18:51:36 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Investments::StockTransfer;
use strict;
use Portal::Accounting::Objects::Investment;
use Portal::Accounting::Objects::ChartOfAccounts;
use Portal::Accounting::Objects::Stock;
use Portal::Accounting::Objects::SubEntity;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Investment Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.3 $';

=head1 NAME

StockTransfer - Object used to create, edit, save an StockTransfer.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::StockTransfer;
  my $obj = Portal::Accounting::Objects::StockTransfer->new();
  $obj->populate(
    name => "test", type => 1, code => "1234", );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

StockTransfer is a StockTransfer class.  This represents a sub account in the
accounting system.  Right now number is used internally. Name and code
are user defined.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::StockTransfer module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "sxfr";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", -Value => "", -DataType => "int" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $self->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -WidgetOptions => { year => $date[0] }, -Value => $today },
    "account" => { -Label => $l->map("From"), -Type => "select", -DataType => "int",
      -Options => $self->{accounts}      },
    "account2" => { -Label => $l->map("To")." ".$l->map("Account"), -Type => "select", -DataType => "int",
      -Options => $self->{accounts}      },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks}      },
    "lots" => { -Type => "hidden", -DataType => "text" },
    "subEntity" => { -Label => $l->map("With")." ".$l->map("Sub Entity"), -Type => "select", -DataType => "int",
      -Options => $self->{subEntities}, -onload => "this.selectedIndex = -1;"      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
    "shares" => { -Type => "text", -DataType => "float", -Label => $l->map("Shares"), size => 8},
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $SelectLots = $l->map("SelectLots");
  my $Submit = $l->map("Submit");
  my $Transfer = $l->map("Transfer");
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=id#
  #FIELD=ledgerCode#
  <h4>#LABEL=type##FIELD=type#</h4>
  <table class="main">
    <tr>
      <td>
        #LABEL=date# #FIELD=date#<br>
        <br>#LABEL=account# #FIELD=account# #FIELD=stock# #FIELD=lots#
        <input type="button" name="lotsButton" value="*"
        onClick="window.open('?app=$app&state=$state&ledgerCode=$lc&command=shareSelect&account=' + this.form.account.options[this.form.account.selectedIndex].value + '&stock=' + this.form.stock.options[this.form.stock.selectedIndex].value + '&date=' + this.form.date.value, '$lc sharesselection', 'toolbars=no,location=no,status=no,width=740,height=500,resizable,scrollbars');" title="$SelectLots"><br>
        $Transfer
        #FIELD=shares# #LABEL=shares#<br>
        #LABEL=account2# #FIELD=account2#
        #LABEL=subEntity# #FIELD=subEntity#<br>
        #LABEL=comment# #FIELD=comment#
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date shares stock account account2 ) ],
    optional => [ qw( lots subEntity ) ],
    constraints => {
       },
  };

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
