# Search.pm - The Object Class that provides a Search Object
#$Id: Search.pm,v 1.5 2005/08/16 22:05:21 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Search;
use strict;
use Portal::Accounting::Objects::BaseL;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Objects::Stock;
use Portal::Accounting::Objects::SubEntity;
use Portal::Accounting::Objects::Ledger;
use Portal::Accounting::Objects::JournalEntry;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.5 $';

=head1 NAME

Search - Object used to build a Search Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Search;
  my $SearchObj = Portal::Accounting::Objects::Search->new(   );
  if ($SearchObj->error())
  {
    die $SearchObj->errorMessage();
  }

=head1 DESCRIPTION

Search is a Search class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Search.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  $self->{object} = "Portal::Accounting::Objects::JournalEntry";
  my @date = Today();
  my $l = $self->{langObj};
  my $between = $l->map('between');
  my $acc = Portal::Accounting::Objects::Account->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $sacc = Portal::Accounting::Objects::SubAccount->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $st = Portal::Accounting::Objects::Stock->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $se = Portal::Accounting::Objects::SubEntity->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $lo = Portal::Accounting::Objects::Ledger->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  $lo->set($lo->{id}, $args{ledgerCode});
  $lo->select;
#  if ($lo->error) { $self->error("<p>Error during ledger object creation!</p>".$lo->errorMessage."\n"); return $self; }
  my $je = Portal::Accounting::Objects::JournalEntry->new(langObj => $l, dbHandle => $self->{dbHandle}, ledger => $lo, ledgerCode => $args{ledgerCode});
#  if ($sacc->error) { $self->error("<p>Error during object creation!</p>".$sacc->errorMessage."\n"); return $self; }
  my $Check = $l->map('Check');
  my $Deposit = $l->map('Deposit');
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "JournalEntry" },
    "command" => { -Type => "hidden", -Value => "search" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "payees" => { -Type => "multi-select", -DataType => "int",
      -Options => $je->selectPayeeOptions(), size => 4, -onload => 'this.selectedIndex = -1;',
    },
    "description" => { -Type => "text", -DataType => "string", },
    "subEntities" => { -Label => $l->map("Sub Entity"), -Type => "multi-select", -DataType => "int",
      -Options => $se->selectOptions, size => 4,
    },
    "accounts" => { -Label => $l->map("Account"), -Type => "multi-select", -DataType => "int",
      -Options => $acc->selectOptions(), size => 4,
    },
    "subAccounts" => { -Label => $l->map("Sub Account"), -Type => "multi-select", -DataType => "int",
      -Options => $sacc->selectOptionsAllByType(), size => 4,
    },
    "stocks" => { -Label => $l->map("Stocks"), -Type => "multi-select", -DataType => "int",
      -Options => $st->selectOptions(), size => 4,
    },
    "number" => { -Type => "select", -DataType => "string",
      -Options => { names => [ qw( all 10 30 ) ], values => [ qw( all 10 30 ) ]},
    },
    "sqlOperator" => { -Type => "select", -DataType => "string",
      -Options => { names => [ qw( AND OR ) ], values => [ qw( AND OR ) ]},
    },
    "dateOperator1" => { -Type => "select", -DataType => "string",
      -Options => { names => [ qw( = < <= > >= ) ],
      values => [ qw( = < <= > >= ) ]},
    },
    "dateOperator2" => { -Type => "select", -DataType => "string",
      -Options => { names => [ qw( = < <= > >= ) ],
      values => [ qw( = < <= > >= ) ]},
    },
    "from" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -WidgetOptions => { year => $date[0] },
    },
    "to" => { -Label => $l->map("Date"),   -Type => "date-picker", -DataType => "string",
      -WidgetOptions => { year => $date[0] },
    },
    "dorc" => { -Type => "radio", -DataType => 'string', -Value => 'false',
      -Options => { names => [ $Check, $Deposit ], values => [ qw( false true ) ]},
    },
    "checkDeposit" => { -Label => $l->map("Number"), -Type => "text",
      size => 8, -DataType => "int",
    },
    "comment" => { -Label => $l->map("Comment"), -Type => "text",
      size => 25, -DataType => "string",
    },
    "exact" => { -Type => "checkbox", -DataType => 'integer',
      -Value => 1, -boolean => 1, -Label => $l->map('Exact Match'),
      #should comment be a substring or exactly what is typed by user
    },
    "id" => { -Label => $l->map("Id"), -Type => "text", size => 6,
      -DataType => "int",
    },
    "amount1" => {-Type => "calculator", size => 10,
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
    "amount2" => {-Type => "calculator", size => 10,
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
    "amountOperator1" => { -Type => "select", -DataType => "string",
      -Options => { names => [ qw( = < <= > >= ) ],
      values => [ qw( = < <= > >= ) ]},
    },
    "amountOperator2" => { -Type => "select", -DataType => "string",
      -Options => { names => [ qw( = < <= > >= ) ],
      values => [ qw( = < <= > >= ) ]},
    },
    "page" => { -Type => "hidden", -DataType => "int", -Value => 1,},
  };
   $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=page#
  <table class="main" cellpadding="5">
    <tr>
      <td colspan="2">
        #LABEL=from#
        #FIELD=dateOperator1# #FIELD=from#
        #LABEL=to#
        #FIELD=dateOperator2# #FIELD=to#
      </td>
    </tr>
    <tr>
      <td>#LABEL=id#
      </td>
      <td>#FIELD=id#
      </td>
    </tr>
    <tr>
      <td>#FIELD=dorc#
      </td>
      <td>#FIELD=checkDeposit#
      </td>
    </tr>
    <tr>
      <td>
        #LANG=Amounts#
      </td>
      <td>
        #FIELD=amountOperator1# #FIELD=amount1#
        #FIELD=amountOperator2# #FIELD=amount2#
      </td>
    </tr>
    <tr>
      <td>#LABEL=comment#
      </td>
      <td>#FIELD=comment# #FIELD=exact# #LABEL=exact#
      </td>
    </tr>
    <tr>
      <td>#LANG=Payee#/<br>#LANG=Description#
      </td>
      <td>
        <table border="0" cellpadding="10" width="300">
          <tr>
            <td>#FIELD=payees#</td>
            <td>#LANG=substringMatch#<br> #FIELD=description#</td>
          </tr>
        </table>
      </td>
    </tr>
    <tr><td colspan="2">
      <table width="100%">
        <tr>
          <td>
            #LABEL=subEntities#<br>
            #FIELD=subEntities#
          </td>
          <td>
            #LABEL=accounts#<br>
            #FIELD=accounts#
          </td>
        </tr>
      </table>
    </td></tr>
    <tr><td colspan="2">
      <table width="100%">
        <tr>
          <td>
            #LABEL=subAccounts#<br>
            #FIELD=subAccounts#
          </td>
          <td>
            #LABEL=stocks#<br>
            #FIELD=stocks#
          </td>
        </tr>
      </table>
    </td></tr>
    <tr>
      <td>#LANG=Transactions#<br>#LANG=per page# #FIELD=number#
      </td align="center">
      <td>#LANG=sql search operator# #FIELD=sqlOperator#
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" />
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
function submitWrapper()
{
  document.main.submit();
}

ENDOF
  $self->{profile}->{main} = {
    required => [ qw( sqlOperator number dateOperator1 dateOperator2 amountOperator1 amountOperator2 ) ],
    optional => [ qw( dorc from to id checkDeposit amount1 amount1 comment payees description subAccounts accounts subEntities stocks) ],
    constraints => {
      id => qr/^\d+$/,
      amount1 => qr/^\d+\.?\d*$/,
      amount2 => qr/^\d+\.?\d*$/,
      from => qr/^\d{4}-\d{2}-\d{2}$/,
      to => qr/^\d{4}-\d{2}-\d{2}$/,
      checkDeposit => qr/^\d+$/,
      dateOperator1 => qr/^[<>=]+$/,
      dateOperator2 => qr/^[<>=]+$/,
      amountOperator1 => qr/^[<>=]+$/,
      amountOperator2 => qr/^[<>=]+$/,
      accounts => qr/^\d+(\0\d+)*$/,
      subAccounts => qr/^\d+(\0\d+)*$/,
      stocks => qr/^\d+(\0\d+)*$/,
      dorc => qr/^true|false$/,
      sqlOperator => qr/^AND|OR$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

sub isNew
{
  return 1;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub selectSQL
{
  my $self = shift;
  my @sql;
  $self->{LIST}[0] = "SELECT DISTINCT(journal_entry_tb.ref_num) AS id, journal_entry_tb.trans_date FROM journal_entry_tb, debitcredit_tb ";
  $self->{LIST}[0] .= "WHERE journal_entry_tb.ledger_code = '".$self->get('ledgerCode')."' AND journal_entry_tb.ref_num = debitcredit_tb.parent_num ";
  my $from = $self->get('from');
  my $to = $self->get('to');
  my $dateOp1 = $self->get('dateOperator1');
  my $dateOp2 = $self->get('dateOperator2');
  if ($from ne '')
  {
    push( @sql , " journal_entry_tb.trans_date $dateOp1 '$from' ");
  }
  if ($to ne '')
  {
    push @sql , " journal_entry_tb.trans_date $dateOp2 '$to' ";
  }
  my $id = $self->get('id');
  if ($id ne '')
  {
    push( @sql , " journal_entry_tb.ref_num = '$id' ");
  }
  my $payees = $self->get('payees');
  if ($payees ne '')
  {
    $payees =~ s/'/\\'/g;
    my @payees = split("\0", $payees);
    my $payee = join("', '", @payees);
    push @sql , " journal_entry_tb.payee_payer IN ('$payee') ";
  }
  my $description = $self->get('description');
  if ($description ne '')
  {
    $description =~ s/_/\\_/g;
    $description =~ s/\%/\\%/g;
    $description =~ s/'/\\'/g;
    push @sql , " journal_entry_tb.payee_payer LIKE '%$description%' ";
  }
  my $checkDeposit = $self->get('checkDeposit');
  if ($checkDeposit ne '')
  {
    my $dorc = $self->get('dorc');
    push @sql , " (debitcredit_tb.dorc = '$dorc' AND debitcredit_tb.sub_account_num = '$checkDeposit' AND (SELECT type FROM account_tb WHERE number = debitcredit_tb.account_num LIMIT 1) = 0 )";
  }
  my $amount1 = $self->get('amount1');
  my $amount2 = $self->get('amount2');
  my $aOp1 = $self->get('amountOperator1');
  my $aOp2 = $self->get('amountOperator2');
  if ($amount1 ne '')
  {
    push @sql , " debitcredit_tb.amount $aOp1 '$amount1' ";
  }
  if ($amount2 ne '')
  {
    push @sql , " debitcredit_tb.amount $aOp2 '$amount2' ";
  }
  my $subEntities = $self->get('subEntities');
  if ($subEntities ne '')
  {
    $subEntities =~ s/'/\\'/g;
    my @subEntities = split("\0", $subEntities);
    my $subEntitie = join("', '", @subEntities);
    push @sql , " debitcredit_tb.sub_e_code IN ('$subEntitie') ";
  }
  my $accounts = $self->get('accounts');
  if ($accounts ne '')
  {
    $self->debug('account');
    my @accounts = split("\0", $accounts);
    my $account = join(", ", @accounts);
    push @sql , " debitcredit_tb.account_num IN ($account) ";
  }
  my $subAccounts = $self->get('subAccounts');
  if ($subAccounts ne '')
  {
    $self->debug('subAcc');
    my @subAccounts = split("\0", $subAccounts);
    my $subAccount = join(", ", @subAccounts);
    push @sql , " debitcredit_tb.sub_account_num IN ($subAccount) ";
  }
  $subAccounts = $self->get('stocks');
  if ($subAccounts ne '')
  {
    $self->debug('stocks');
    my @subAccounts = split("\0", $subAccounts);
    my $subAccount = join(", ", @subAccounts);
    push @sql , " debitcredit_tb.sub_account_num IN ($subAccount) ";
  }
  my $comment = $self->get('comment');
  if ($comment ne '')
  {
    $comment =~ s/'/\\'/g;
    if ($self->{data}->{main}->{exact}->{checked})
    {
      push @sql , " debitcredit_tb.comment = '$comment' ";
    }
    else
    {
      $comment =~ s/_/\\_/g;
      $comment =~ s/\%/\\%/g;
      push @sql , " debitcredit_tb.comment LIKE '%$comment%' ";
    }
  }
  if (scalar @sql > 0)
  {
    $self->{LIST}[0] .= " AND ( ". join(" ".$self->get('sqlOperator')." ", @sql) . ") ";
  }
  $self->{LIST}[0] .= " ORDER BY trans_date ";
  my $num = $self->get('number');
  if ($num ne 'all')
  {
    $self->{LIST}[0] .= " LIMIT $num";
    my $page = $self->get('page');
    if ($page > 1)
    {
      $self->{LIST}[0] .= " OFFSET " . ($num * ($page - 1)) + 1;
    }
  }
  return $self->{LIST}[0];
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
