# SubEntity.pm - The Object Class that provides a SubEntity Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: SubEntity.pm,v 1.17 2005/08/11 19:40:57 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::SubEntity;
use strict;
use Portal::Accounting::Objects::BaseL;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.17 $';

=head1 NAME

SubEntity - Object used to create, edit, save an SubEntity.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::SubEntity;
  my $obj = Portal::Accounting::Objects::SubEntity->new();
  $obj->populate(
    name => "test", type => 1, code => "1234", );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

SubEntity is a SubEntity class.  This represents a sub account in the
accounting system.  Right now number is used internally. Name and code
are user defined.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::SubEntity module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", code => "", app => "Accounting", state => "SubEntities", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $l = $self->{langObj};
  $self->{table} = "sub_entity_tb";
  my $Submit = $l->map("Save");
  my $codeInUse = $l->map("codeInUse");
  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "embed" => { -Type => "hidden", -Value => "0", -DataType => "int" },
    "new" => { -Type => "hidden", -Value => "1", -DataType => "int" },
    "name" => { -Label => $l->map("Name"), -Type => "text", size => 15, -DataType => "string", -onload => "this.select();" },
    "code" => { -Label => $l->map("Code"), -Type => "text", size => 5, -DataType => "string"},
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=embed#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=new#
  #FIELD=ledgerCode#
  <table class="main">
    <tr>
      <td>#LABEL=name#</td><td>#FIELD=name#</td>
    </tr><tr>
      <td>#LABEL=code#</td><td>#FIELD=code#</td>
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code ledgerCode ) ],
    optional => [  ],
    constraints => {
      name => qr/^.+$/,
      code => qr/^.+$/,
       },
  };
  $self->{defaultForm} = "main";
  ##db infrastructure
  $self->{id} = "code";
  $self->{INSERTorder}[0] = [ qw( name code ledger_code ) ] ;  #insert order for db column names
  $self->{newOnly}[0] = { code => "", ledger_code => "" };
  $self->{db2data}[0] = { ledger_code => 'ledgerCode' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = reverse %{$self->{db2data}[0]};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM sub_entity_tb WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";
  $self->{INSERT}[0] = "INSERT INTO sub_entity_tb (##COLUMNS##) VALUES (##VALUES##)";
  $self->{preINSERT}[0] = "SELECT '$codeInUse'||name from sub_entity_tb WHERE ledger_code = '#FIELD=ledgerCode#' AND code = '#FIELD=code#'";
  $self->{UPDATE}[0] = "UPDATE sub_entity_tb SET ##UPDATE## WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[0] = "DELETE FROM sub_entity_tb WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";
  #what should we output as the value of the thing just deleted?
  $self->{postDELETE}[0] = "SELECT name||', '||code FROM sub_entity_tb WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";
  $self->{LIST}[0] = "SELECT name||' ('||code||')' AS name, code FROM sub_entity_tb WHERE ledger_code = '#FIELD=ledgerCode#' ORDER BY code";

  return $self;
}

sub isNew
{
  my $self = shift;
  return $self->get('new');
}

sub select
{
  my $self = shift;
  $self->{data}->{main}->{new}->{-Value} = 0;
  return $self->SUPER::select(@_);
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub printHTMLHeader
{
  my $self = shift;
  my $Name = $self->{langObj}->map("Name");
  my $Code = $self->{langObj}->map("Code");
  my $Delete = $self->{langObj}->map("Delete");
  my $return = <<"ENDOF";
  <tr class="edit">
    <th class="name">$Name</th>
    <th class="code">$Code</th>
    <th class="delete">$Delete</th>
  </tr>
ENDOF
  return $return;
}

sub printHTMLRow
{
  my $self = shift;
  my $Edit = $self->{langObj}->map("Edit");
  my $Delete = $self->{langObj}->map("Delete");
  my $name = $self->get('name');
  my $code = $self->get('code');
  my $l = $self->{data}->{main}->{ledgerCode}->{-Value};
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $return = "  <tr>\n";
  $return .= <<"ENDOF";
    <td class="name">
      <a title="$Edit" href="?app=$app&state=$state&command=edit&ledgerCode=$l&code=$code">$name</a>
    </td>
    <td class="code">$code&nbsp;</td>
    <td class="delete">
      <form name="delete_$code">
        <input type="hidden" name="app" value="$app">
        <input type="hidden" name="state" value="$state">
        <input type="hidden" name="ledgerCode" value="$l">
        <input type="hidden" name="command" value="delete">
        <input type="hidden" name="code" value="$code">
        <input type="hidden" name="confirmDelete" value="0">
        <input type="submit" name="delete" value="X" title="$Delete" onclick="return deleteConfirm(this.form); ">
      </form>
    </td>
ENDOF
  $return .= "  </tr>\n";
  return $return;
}

=item javascriptOverhead(ledgerCode)

  this routine returns a string with all of the javascript overhead needed
  for managing sub entities

=cut
sub javascriptOverhead
{
  my $self = shift;
  my $lc = shift;
  my $j;
  my $sql = "SELECT name, code FROM sub_entity_tb WHERE ledger_code = '$lc'";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  while (my $d = $result->fetchrow_hashref)
  {
    $j .= "SUBENTITIES['$d->{code}'] = '$d->{name}';\n";
  }
  return $j."\n";
}

sub update
{
  my $self = shift;
  if ($self->get("name") eq "") { $self->invalid('name', ''); $self->error('invalid name'); return undef; }
  return $self->SUPER::update(@_);
}

sub delete
{
  my $self = shift;
  # check any prerequisites that should stop delete from happening!  output better error messages
  my $sql = "SELECT debitcredit_tb.parent_num AS id FROM debitcredit_tb WHERE debitcredit_tb.sub_e_code = '".$self->get('code')."'";
  my $sth = $self->dbHandle->read($sql);
  my $n = 0;
  while (my $r = $sth->fetchrow_hashref)
  {
    $n++;
    $self->{result} .= $self->langObj->map('Transaction').' '.$r->{id}.' '.$self->langObj->map("is using this sub entity")."<br>\n";
  }
  if ($n > 0) { return -1; }
  return $self->SUPER::delete(@_);
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
