# SubEntities.pm - The Object Class that provides a SubEntities Object
#$Id: SubEntities.pm,v 1.10 2005/08/08 22:18:36 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::SubEntities;
use strict;
use Portal::Accounting::BaseL;
use Portal::Accounting::Objects::SubEntity;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.10 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Sub Entities",
  new => "New Sub Entity",
  save => "Save Sub Entity",
  edit => "Edit Sub Entity",
  delete => "Delete Sub Entity",
  );

=head1 NAME

SubEntities - Object used to work with SubEntities

=head1 SYNOPSIS

  use Portal::Accounting::SubEntities;
  my $obj = Portal::Accounting::SubEntities->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

SubEntities displays the SubEntities Menu from which sub Entities
can be edited, created, and deleted.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::SubEntities module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the SubEntities State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;
  $self->{object} = "SubEntity";
  $self->{css} .= <<"ENDOF";
  .name { text-align: left; }
  .code { text-align: left; }
  .delete { text-align: center; }
ENDOF

  # do validation
  if (!$self->Portal::Accounting::SubEntities::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub c_save
{
  my $self = shift;
  my $doc = $self->SUPER::c_save(@_);
  my $name = $self->{OBJECT}->get("name");
  my $code = $self->{OBJECT}->get("code");
  if ($self->{input}->{embed} == 1)
  {
    $doc->setOnload( <<"ENDOF" );
  var $self->{object} = new Object();
  $self->{object}.name = '$name';
  $self->{object}.code = '$code';
  if (opener)
  {
    if (opener.newSubAccountData) opener.newSubEntityData($self->{object});
    else { alert("function not found"); }
  }
ENDOF
  }
  return $doc;
}

sub c_edit
{
  my $self = shift;
  my $doc = $self->SUPER::c_edit(@_);
  if (! defined $self->{input}->{embed} || $self->{input}->{embed} == 0)
  {
    $doc->print($self->mainButton);
  }
  elsif (defined $self->{object})
  {
    $doc->print($self->{object}->javascript);
  }
  return $doc;
}

sub c_delete
{
  my $self = shift;
  my $doc = $self->SUPER::c_delete(@_);
  if (! defined $self->{input}->{embed} || $self->{input}->{embed} == 0)
  {
    $doc->print($self->mainButton);
  }
  elsif (defined $self->{object})
  {
    $doc->print($self->{object}->javascript);
  }
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
